@@ -1,6 +1,6 @@
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2011-2013 Andreas Rönnquist
+ *	Copyright (C) 2011-2015 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -68,7 +68,7 @@ int c_get_window_name(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_name: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -76,7 +76,7 @@ int c_get_window_name(lua_State *lua)
 
 	WnckWindow *window = get_current_window();
 	if (window) {
-		test=(char*)wnck_window_get_name(window);
+		test = (char*)wnck_window_get_name(window);
 	} else {
 		test = "";
 	}
@@ -98,7 +98,7 @@ int c_get_window_has_name(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_has_name: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -125,7 +125,7 @@ int c_set_window_geometry(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=4) {
+	if (top != 4) {
 		luaL_error(lua, "set_window_geometry: %s", four_indata_expected_error);
 		return 0;
 	}
@@ -135,10 +135,10 @@ int c_set_window_geometry(lua_State *lua)
 	int type3 = lua_type(lua, 3);
 	int type4 = lua_type(lua, 4);
 
-	if ((type1!=LUA_TNUMBER) ||
-	    (type2!=LUA_TNUMBER) ||
-	    (type3!=LUA_TNUMBER) ||
-	    (type4!=LUA_TNUMBER)) {
+	if ((type1 != LUA_TNUMBER) ||
+	        (type2 != LUA_TNUMBER) ||
+	        (type3 != LUA_TNUMBER) ||
+	        (type4 != LUA_TNUMBER)) {
 		luaL_error(lua, "set_window_geometry: %s", four_indata_expected_error);
 		return 0;
 	}
@@ -151,33 +151,7 @@ int c_set_window_geometry(lua_State *lua)
 	if (!devilspie2_emulate) {
 		WnckWindow *window = get_current_window();
 
-		if (window) {
-			WnckScreen *screen = wnck_window_get_screen(window);
-			int sw = wnck_screen_get_width(screen);
-			int sh = wnck_screen_get_height(screen);
-
-			int gravity = WNCK_WINDOW_GRAVITY_CURRENT;
-			if (x >= 0 && y >= 0)
-				gravity = WNCK_WINDOW_GRAVITY_NORTHWEST;
-			if (x >= 0 && y < 0)
-				gravity = WNCK_WINDOW_GRAVITY_SOUTHWEST;
-			if (x < 0 && y >= 0)
-				gravity = WNCK_WINDOW_GRAVITY_NORTHEAST;
-			if (x < 0 && y < 0)
-				gravity = WNCK_WINDOW_GRAVITY_SOUTHEAST;
-			if (x < 0)
-				x = sw + x;
-			if (y < 0)
-				y = sh + y;
-
-			wnck_window_set_geometry(window,
-			                         gravity,
-			                         WNCK_WINDOW_CHANGE_X +
-			                         WNCK_WINDOW_CHANGE_Y +
-			                         WNCK_WINDOW_CHANGE_WIDTH +
-			                         WNCK_WINDOW_CHANGE_HEIGHT,
-			                         x, y, xsize, ysize);
-		}
+		set_window_geometry(window, x, y, xsize, ysize);
 	}
 
 	return 0;
@@ -192,8 +166,8 @@ int c_set_window_geometry2(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=4) {
-		luaL_error(lua,"set_window_geometry: %s",four_indata_expected_error);
+	if (top != 4) {
+		luaL_error(lua,"set_window_geometry: %s", four_indata_expected_error);
 		return 0;
 	}
 
@@ -202,10 +176,10 @@ int c_set_window_geometry2(lua_State *lua)
 	int type3 = lua_type(lua, 3);
 	int type4 = lua_type(lua, 4);
 
-	if ((type1!=LUA_TNUMBER) ||
-	    (type2!=LUA_TNUMBER) ||
-	    (type3!=LUA_TNUMBER) ||
-	    (type4!=LUA_TNUMBER)) {
+	if ((type1 != LUA_TNUMBER) ||
+	        (type2 != LUA_TNUMBER) ||
+	        (type3 != LUA_TNUMBER) ||
+	        (type4 != LUA_TNUMBER)) {
 		luaL_error(lua, "set_window_geometry: %s", four_indata_expected_error);
 		return 0;
 	}
@@ -221,8 +195,8 @@ int c_set_window_geometry2(lua_State *lua)
 		if (window) {
 			XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
 			                  wnck_window_get_xid(window),
-			                  x,y,
-			                  xsize,ysize);
+			                  x, y,
+			                  xsize, ysize);
 		}
 	}
 
@@ -245,7 +219,7 @@ int c_set_window_position(lua_State *lua)
 	int type1 = lua_type(lua, 1);
 	int type2 = lua_type(lua, 2);
 
-	if ((type1!=LUA_TNUMBER) || (type2!=LUA_TNUMBER)) {
+	if ((type1 != LUA_TNUMBER) || (type2 != LUA_TNUMBER)) {
 		luaL_error(lua, "set_window_position: %s", two_indata_expected_error);
 		return 0;
 	}
@@ -259,9 +233,9 @@ int c_set_window_position(lua_State *lua)
 
 		if (window) {
 			wnck_window_set_geometry(window,
-				WNCK_WINDOW_GRAVITY_CURRENT,
-				WNCK_WINDOW_CHANGE_X + WNCK_WINDOW_CHANGE_Y,
-				x, y, -1, -1);
+			                         WNCK_WINDOW_GRAVITY_CURRENT,
+			                         WNCK_WINDOW_CHANGE_X + WNCK_WINDOW_CHANGE_Y,
+			                         x, y, -1, -1);
 		}
 	}
 
@@ -276,15 +250,15 @@ int c_set_window_position2(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=2) {
+	if (top != 2) {
 		luaL_error(lua,"set_window_position2: %s", two_indata_expected_error);
 		return 0;
 	}
 
-	int type1 = lua_type(lua,1);
-	int type2 = lua_type(lua,2);
+	int type1 = lua_type(lua, 1);
+	int type2 = lua_type(lua, 2);
 
-	if ((type1!=LUA_TNUMBER) || (type2!=LUA_TNUMBER)) {
+	if ((type1 != LUA_TNUMBER) || (type2 != LUA_TNUMBER)) {
 		luaL_error(lua,"set_window_position2: %s", two_indata_expected_error);
 		return 0;
 	}
@@ -299,7 +273,7 @@ int c_set_window_position2(lua_State *lua)
 		if (window) {
 			XMoveWindow(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()),
 			            wnck_window_get_xid(window),
-			            x,y);
+			            x, y);
 		}
 	}
 
@@ -314,15 +288,15 @@ int c_set_window_size(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=2) {
+	if (top != 2) {
 		luaL_error(lua,"set_window_size: %s", two_indata_expected_error);
 		return 0;
 	}
 
-	int type1 = lua_type(lua,1);
-	int type2 = lua_type(lua,2);
+	int type1 = lua_type(lua, 1);
+	int type2 = lua_type(lua, 2);
 
-	if ((type1!=LUA_TNUMBER) || (type2!=LUA_TNUMBER)) {
+	if ((type1 != LUA_TNUMBER) || (type2 != LUA_TNUMBER)) {
 		luaL_error(lua,"set_window_size: %s", two_indata_expected_error);
 		return 0;
 	}
@@ -338,13 +312,13 @@ int c_set_window_size(lua_State *lua)
 
 			devilspie2_error_trap_push();
 			wnck_window_set_geometry(window,
-				WNCK_WINDOW_GRAVITY_CURRENT,
-				WNCK_WINDOW_CHANGE_WIDTH + WNCK_WINDOW_CHANGE_HEIGHT,
-				-1, -1, x, y);
+			                         WNCK_WINDOW_GRAVITY_CURRENT,
+			                         WNCK_WINDOW_CHANGE_WIDTH + WNCK_WINDOW_CHANGE_HEIGHT,
+			                         -1, -1, x, y);
 
 			if (devilspie2_error_trap_pop()) {
 				gchar *temperror=
-					g_strdup_printf("set_window_size: %s", failed_string);
+				    g_strdup_printf("set_window_size: %s", failed_string);
 				g_printerr("%s", temperror);
 
 				g_free(temperror);
@@ -368,11 +342,11 @@ int c_set_window_strut(lua_State *lua)
 	}
 
 	if (!devilspie2_emulate) {
-		#if __x86_64__
+#if __x86_64__
 		int64_t struts[12];
-		#else
+#else
 		int32_t struts[12];
-		#endif
+#endif
 		int i;
 		for (i = 0; i < 12; i++) {
 			struts[i] = i < top ? lua_tonumber(lua, i + 1) : 0;
@@ -383,12 +357,12 @@ int c_set_window_strut(lua_State *lua)
 
 		if (window) {
 			XChangeProperty(dpy,
-							wnck_window_get_xid(window),
-							XInternAtom(dpy, "_NET_WM_STRUT_PARTIAL", False), XA_CARDINAL,
-							32,
-							PropModeReplace,
-							(unsigned char*)struts,
-							12);
+			                wnck_window_get_xid(window),
+			                XInternAtom(dpy, "_NET_WM_STRUT_PARTIAL", False), XA_CARDINAL,
+			                32,
+			                PropModeReplace,
+			                (unsigned char*)struts,
+			                12);
 			XSync(dpy, False);
 		}
 	}
@@ -403,7 +377,7 @@ int c_make_always_on_top(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "make_always_on_top: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -427,7 +401,7 @@ int c_set_on_top(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "set_on_top: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -453,7 +427,7 @@ int c_get_application_name(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_application_name: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -465,7 +439,7 @@ int c_get_application_name(lua_State *lua)
 	if (window) {
 
 		WnckApplication *application=
-			wnck_window_get_application(get_current_window());
+		    wnck_window_get_application(get_current_window());
 		application_name = wnck_application_get_name(application);
 
 	} else {
@@ -489,7 +463,7 @@ int c_debug_print(lua_State *lua)
 	int n = lua_gettop(lua);  /* number of arguments */
 	int i;
 	lua_getglobal(lua, "tostring");
-	for (i=1; i<=n; i++) {
+	for (i = 1; i <= n; i++) {
 		const char *s;
 		lua_pushvalue(lua, -1);  /* function to be called */
 		lua_pushvalue(lua, i);   /* value to print */
@@ -497,14 +471,17 @@ int c_debug_print(lua_State *lua)
 		s = lua_tostring(lua, -1);  /* get result */
 		if (s == NULL)
 			return luaL_error(lua, LUA_QL("tostring") " must return a string to "
-									LUA_QL("print"));
-		if (i>1) {
+			                  LUA_QL("print"));
+		if (i > 1) {
 			if (devilspie2_debug) fputs("\t", stdout);
 		}
 		if (devilspie2_debug) fputs(s, stdout);
 		lua_pop(lua, 1);  /* pop result */
 	}
-	if (devilspie2_debug) fputs("\n", stdout);
+	if (devilspie2_debug) {
+		fputs("\n", stdout);
+		fflush(stdout);
+	}
 
 	return 0;
 }
@@ -517,7 +494,7 @@ int c_shade_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua,"shade_window: %s",no_indata_expected_error);
 		return 0;
 	}
@@ -542,7 +519,7 @@ int c_unshade_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua,"unshade_window: %s");
 		return 0;
 	}
@@ -567,13 +544,13 @@ int c_minimize_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua,"minimize_window: %s",no_indata_expected_error);
 		return 0;
 	}
 
 	if (!devilspie2_emulate) {
-				WnckWindow *window = get_current_window();
+		WnckWindow *window = get_current_window();
 
 		if (window) {
 
@@ -592,7 +569,7 @@ int c_unminimize_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua,"unminimize_window: %s",no_indata_expected_error);
 		return 0;
 	}
@@ -632,10 +609,10 @@ WnckWindow *get_current_window()
  */
 int c_undecorate_window(lua_State *lua)
 {
-	gboolean result=TRUE;
+	gboolean result = TRUE;
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua,"undecorate_window: %s",no_indata_expected_error);
 		return 0;
 	}
@@ -666,10 +643,10 @@ int c_undecorate_window(lua_State *lua)
  */
 int c_decorate_window(lua_State *lua)
 {
-	gboolean result=TRUE;
+	gboolean result = TRUE;
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua,"decorate_window: %s",no_indata_expected_error);
 		return 0;
 	}
@@ -704,7 +681,7 @@ int c_set_window_workspace(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua,"set_window_workspace: %s", one_indata_expected_error);
 		return 0;
 	}
@@ -713,7 +690,7 @@ int c_set_window_workspace(lua_State *lua)
 
 	if (type!=LUA_TNUMBER) {
 		luaL_error(lua, "set_window_workspace: %s",
-			number_expected_as_indata_error);
+		           number_expected_as_indata_error);
 		return 0;
 	}
 
@@ -752,12 +729,12 @@ int c_change_workspace(lua_State *lua)
 	int top = lua_gettop(lua);
 	GTimeVal timestamp;
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua,"change_workspace: %s", one_indata_expected_error);
 		return 0;
 	}
 
-	int type=lua_type(lua, 1);
+	int type = lua_type(lua, 1);
 
 	if (type!=LUA_TNUMBER) {
 		luaL_error(lua,"change_workspace: %s", number_expected_as_indata_error);
@@ -798,8 +775,8 @@ int c_get_workspace_count(lua_State *lua)
 	int count = 0;
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"get_workspace_count: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "get_workspace_count: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -825,8 +802,8 @@ int c_unmaximize_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"unmaximize: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "unmaximize: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -848,8 +825,8 @@ int c_maximize_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"maximize: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "maximize: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -870,8 +847,8 @@ int c_maximize_window_vertically(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"maximize_vertically: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "maximize_vertically: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -893,8 +870,8 @@ int c_maximize_window_horisontally(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"maximize_horisontally: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "maximize_horisontally: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -917,8 +894,8 @@ int c_pin_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"pin_window: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "pin_window: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -941,8 +918,8 @@ int c_unpin_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"unpin_window: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "unpin_window: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -964,8 +941,8 @@ int c_stick_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"stick_window: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "stick_window: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -987,8 +964,8 @@ int c_unstick_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"unstick_window: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "unstick_window: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -1004,14 +981,14 @@ int c_unstick_window(lua_State *lua)
 
 
 /**
- * return the geometry of the current window to the LUA script
+ * return the geometry of the current window to the Lua script
  */
 int c_get_window_geometry(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"get_window_geometry: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "get_window_geometry: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -1020,7 +997,7 @@ int c_get_window_geometry(lua_State *lua)
 	WnckWindow *window = get_current_window();
 	if (window)
 	{
-		wnck_window_get_geometry(window,&x,&y,&width,&height);
+		wnck_window_get_geometry(window, &x, &y, &width, &height);
 	}
 
 	lua_pushnumber(lua, x);
@@ -1033,15 +1010,15 @@ int c_get_window_geometry(lua_State *lua)
 
 
 /**
- * return the client geometry of the current window to the LUA script
+ * return the client geometry of the current window to the Lua script
  * this is excluding the window manager frame
  */
 int c_get_client_window_geometry(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"get_client_window_geometry: %s",no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "get_client_window_geometry: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -1069,14 +1046,14 @@ int c_set_skip_tasklist(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua, "set_skip_tasklist: %s", one_indata_expected_error);
 		return 0;
 	}
 
 	int type = lua_type(lua, 1);
 
-	if (type!=LUA_TBOOLEAN) {
+	if (type != LUA_TBOOLEAN) {
 		luaL_error(lua, "set_skip_tasklist: %s", boolean_expected_as_indata_error);
 		return 0;
 	}
@@ -1103,26 +1080,26 @@ int c_set_skip_pager(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua, "set_skip_pager: %s", one_indata_expected_error);
 		return 0;
 	}
 
 	int type = lua_type(lua, 1);
 
-	if (type!=LUA_TBOOLEAN) {
+	if (type != LUA_TBOOLEAN) {
 		luaL_error(lua, "set_skip_pager: %s", boolean_expected_as_indata_error);
 		return 0;
 	}
 
 	int value = lua_toboolean(lua, 1);
 
-	gboolean skip_pager=(gboolean)(value);
+	gboolean skip_pager = (gboolean)(value);
 
 	if (!devilspie2_emulate) {
 		WnckWindow *window = get_current_window();
 		if (window) {
-			wnck_window_set_skip_pager(window,skip_pager);
+			wnck_window_set_skip_pager(window, skip_pager);
 		}
 	}
 
@@ -1137,9 +1114,9 @@ int c_get_window_is_maximized(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_is__maximized: %s",
-			no_indata_expected_error);
+		           no_indata_expected_error);
 		return 0;
 	}
 
@@ -1162,9 +1139,9 @@ int c_get_window_is_maximized_vertically(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_is_horisontally_maximized: %s",
-			no_indata_expected_error);
+		           no_indata_expected_error);
 		return 0;
 	}
 
@@ -1188,9 +1165,9 @@ int c_get_window_is_maximized_horisontally(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_is_horisontally_maximized: %s",
-			no_indata_expected_error);
+		           no_indata_expected_error);
 		return 0;
 	}
 
@@ -1210,25 +1187,64 @@ int c_get_window_is_maximized_horisontally(lua_State *lua)
 /**
  *
  */
-int c_set_window_above(lua_State *lua)
+int c_get_window_is_pinned(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"set_window_above: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "get_window_is_pinned: %s",
+		           no_indata_expected_error);
 		return 0;
 	}
 
 	WnckWindow *window = get_current_window();
 
-	if (!devilspie2_emulate) {
+	gboolean is_pinned = FALSE;
+	if (window) {
+		is_pinned = wnck_window_is_pinned(window);
+	}
 
+	lua_pushboolean(lua, is_pinned);
+
+	return 1;
+}
+
+
+/**
+ *
+ */
+int c_set_window_above(lua_State *lua)
+{
+	int top = lua_gettop(lua);
+	gboolean set_above;
+
+	if (top > 1) {
+		luaL_error(lua, "set_window_above: %s", one_indata_expected_error);
+		return 0;
+	}
+	else if (top == 1) {
+		int type = lua_type(lua, 1);
+
+		if (type != LUA_TBOOLEAN) {
+			luaL_error(lua, "set_window_above: %s", boolean_expected_as_indata_error);
+			return 0;
+		}
+
+		int value = lua_toboolean(lua, 1);
+		set_above = (gboolean)(value);
+	}
+	else {
+		set_above = TRUE;
+	}
+
+	WnckWindow *window = get_current_window();
+	if (!devilspie2_emulate) {
 		Window xid = wnck_window_get_xid(window);
 		devilspie2_change_state(devilspie2_window_get_xscreen(xid),
-		                     xid,
-		                     TRUE,
-		                     my_wnck_atom_get("_NET_WM_STATE_ABOVE"),
-		                     0);
+		                        xid,
+		                        set_above,
+		                        my_wnck_atom_get("_NET_WM_STATE_ABOVE"),
+		                        0);
 	}
 
 	return 0;
@@ -1241,22 +1257,36 @@ int c_set_window_above(lua_State *lua)
 int c_set_window_below(lua_State *lua)
 {
 	int top = lua_gettop(lua);
+	gboolean set_below;
 
-	if (top!=0) {
-		luaL_error(lua,"set_window_above: %s", no_indata_expected_error);
+	if (top > 1) {
+		luaL_error(lua, "set_window_below: %s", one_indata_expected_error);
 		return 0;
 	}
+	else if (top == 1) {
+		int type = lua_type(lua, 1);
+
+		if (type != LUA_TBOOLEAN) {
+			luaL_error(lua, "set_window_below: %s", boolean_expected_as_indata_error);
+			return 0;
+		}
+
+		int value = lua_toboolean(lua, 1);
+		set_below = (gboolean)(value);
+	}
+	else {
+		set_below = TRUE;
+	}
 
 	WnckWindow *window = get_current_window();
-
 	if (!devilspie2_emulate) {
 
 		Window xid = wnck_window_get_xid(window);
 		devilspie2_change_state(devilspie2_window_get_xscreen(xid),
-		                     xid,
-		                     TRUE,
-		                     my_wnck_atom_get("_NET_WM_STATE_BELOW"),
-		                     0);
+		                        xid,
+		                        set_below,
+		                        my_wnck_atom_get("_NET_WM_STATE_BELOW"),
+		                        0);
 	}
 
 	return 0;
@@ -1270,8 +1300,8 @@ int c_get_window_type(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
-		luaL_error(lua,"get_window_type: %s", no_indata_expected_error);
+	if (top != 0) {
+		luaL_error(lua, "get_window_type: %s", no_indata_expected_error);
 		return 0;
 	}
 
@@ -1331,9 +1361,9 @@ int c_get_class_instance_name(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_class_instance_name: %s",
-			no_indata_expected_error);
+		           no_indata_expected_error);
 		return 0;
 	}
 
@@ -1364,7 +1394,7 @@ int c_get_window_property(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua, "get_window_property: %s", one_indata_expected_error);
 		return 0;
 	}
@@ -1372,8 +1402,8 @@ int c_get_window_property(lua_State *lua)
 	//	gchar *property=
 	int type = lua_type(lua, 1);
 
-	if (type!=LUA_TSTRING) {
-		luaL_error(lua,"get_window_property: %s",string_expected_as_indata_error);
+	if (type != LUA_TSTRING) {
+		luaL_error(lua, "get_window_property: %s", string_expected_as_indata_error);
 		return 0;
 	}
 
@@ -1385,7 +1415,7 @@ int c_get_window_property(lua_State *lua)
 
 	if (window) {
 		result = my_wnck_get_string_property_latin1(wnck_window_get_xid(window),
-			my_wnck_atom_get(value));
+		         my_wnck_atom_get(value));
 	} else {
 		result = g_strdup_printf("NO RESULT");
 	}
@@ -1409,7 +1439,7 @@ int c_get_window_role(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_role: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -1420,7 +1450,7 @@ int c_get_window_role(lua_State *lua)
 
 	if (window) {
 		result = my_wnck_get_string_property_latin1(wnck_window_get_xid(window),
-			my_wnck_atom_get("WM_WINDOW_ROLE"));
+		         my_wnck_atom_get("WM_WINDOW_ROLE"));
 	}
 
 	if (result) {
@@ -1442,7 +1472,7 @@ int c_get_window_xid(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_xid: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -1470,7 +1500,7 @@ int c_get_window_class(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_window_class: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -1484,12 +1514,12 @@ int c_get_window_class(lua_State *lua)
 
 #ifdef WNCK_MAJOR_VERSION
 #if WNCK_CHECK_VERSION(3,2,0)
-	gchar *class_name=(char*)wnck_class_group_get_id(class_group);
+		gchar *class_name = (char*)wnck_class_group_get_id(class_group);
 #else
-	gchar *class_name=(char*)wnck_class_group_get_res_class (class_group);
+		gchar *class_name = (char*)wnck_class_group_get_res_class (class_group);
 #endif
 #else
-	gchar *class_name = (char*)wnck_class_group_get_res_class (class_group);
+		gchar *class_name = (char*)wnck_class_group_get_res_class (class_group);
 #endif
 
 		result = g_strdup(class_name);
@@ -1513,9 +1543,9 @@ int c_set_window_fullscreen(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua, "set_window_fullscreen: %s",
-			one_indata_expected_error);
+		           one_indata_expected_error);
 		return 0;
 	}
 
@@ -1523,9 +1553,9 @@ int c_set_window_fullscreen(lua_State *lua)
 
 	int type = lua_type(lua, 1);
 
-	if (type!=LUA_TBOOLEAN) {
+	if (type != LUA_TBOOLEAN) {
 		luaL_error(lua, "set_window_fullscreen: %s",
-			boolean_expected_as_indata_error);
+		           boolean_expected_as_indata_error);
 		return 0;
 	}
 
@@ -1548,63 +1578,121 @@ int c_set_window_fullscreen(lua_State *lua)
 int c_set_viewport(lua_State *lua)
 {
 	int top = lua_gettop(lua);
-	WnckScreen *screen;
-	int x,y,width,height, viewport_start;
+	int width, height;
+	int viewport_start_x, viewport_start_y;
+	int win_x, win_y;
+	
+	if (top == 1) {
 
-	if (top!=1) {
-		luaL_error(lua, "set_viewport: %s", one_indata_expected_error);
+		WnckScreen *screen;
+		int x;
+		int type = lua_type(lua, 1);
+		if (type != LUA_TNUMBER) {
+			luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
+			return 0;
+		}
+
+		int num = lua_tonumber(lua,1);
+
+		if (num <= 0) {
+			g_error("set_viewport: %s", integer_greater_than_zero_expected_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		WnckWindow *window = get_current_window();
+
+		if (!window) {
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		screen = wnck_window_get_screen(window);
+
+		wnck_window_get_geometry(window, &win_x, &win_y, &width, &height);
+
+		gulong xid = wnck_window_get_xid(window);
+
+		//viewport_start = devilspie2_get_viewport_start(xid);
+		if (devilspie2_get_viewport_start(xid, &viewport_start_x, &viewport_start_y) != 0) {
+			g_printerr("set_viewport: %s", could_not_find_current_viewport_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		x = ((num - 1) * wnck_screen_get_width(screen)) - viewport_start_x + win_x;
+		
+		devilspie2_error_trap_push();
+		XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
+								wnck_window_get_xid(window),
+								x, win_y, width, height);
+
+		if (devilspie2_error_trap_pop()) {
+			g_printerr("set_viewport: %s", setting_viewport_failed_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		lua_pushboolean(lua, TRUE);
+		return 1;
+
+	} else if (top == 2) {
+		
+		int type1 = lua_type(lua, 1);
+		int type2 = lua_type(lua, 2);
+
+		if (type1 != LUA_TNUMBER) {
+			luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
+			return 0;
+		}
+
+		if (type2 != LUA_TNUMBER) {
+			luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
+			return 0;
+		}
+
+		int new_xpos = lua_tonumber(lua, 1);
+		int new_ypos = lua_tonumber(lua, 2);
+
+		WnckWindow *window = get_current_window();
+
+		if (!window) {
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		wnck_window_get_geometry(window, &win_x, &win_y, &width, &height);
+
+		gulong xid = wnck_window_get_xid(window);
+
+		//viewport_start = devilspie2_get_viewport_start(xid);
+		if (devilspie2_get_viewport_start(xid, &viewport_start_x, &viewport_start_y) != 0) {
+			g_printerr("set_viewport: %s", could_not_find_current_viewport_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+		
+		devilspie2_error_trap_push();
+		XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
+								wnck_window_get_xid(window),
+								new_xpos, new_ypos, width, height);
+
+		if (devilspie2_error_trap_pop()) {
+			g_printerr("set_viewport: %s", setting_viewport_failed_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		lua_pushboolean(lua, TRUE);
+		return 1;
+		
+	
+	} else {
+		luaL_error(lua, "set_viewport: %s", one_or_two_indata_expected_error);
 		return 0;
 	}
-
-	int type=lua_type(lua, 1);
-	if (type!=LUA_TNUMBER) {
-		luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
-		return 0;
-	}
-
-	int num = lua_tonumber(lua,1);
-
-	if (num<=0) {
-		g_error("set_viewport: %s", integer_greater_than_zero_expected_error);
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	WnckWindow *window = get_current_window();
-
-	if (!window) {
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	screen = wnck_window_get_screen(window);
-
-	wnck_window_get_geometry(window, &x, &y, &width, &height);
-
-	gulong xid = wnck_window_get_xid(window);
-
-	viewport_start = devilspie2_get_viewport_start(xid);
-	if (viewport_start<0) {
-		g_printerr("set_viewport: %s", could_not_find_current_viewport_error);
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	x=((num-1) * wnck_screen_get_width(screen)) - viewport_start + x;
-
-	devilspie2_error_trap_push();
-	XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
-	                  wnck_window_get_xid(window),
-	                  x,y,width,height);
-
-	if (devilspie2_error_trap_pop()) {
-		g_printerr("set_viewport: %s",setting_viewport_failed_error);
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	lua_pushboolean(lua, TRUE);
-	return 1;
+	
+	return 0;
 }
 
 
@@ -1620,7 +1708,7 @@ int c_center(lua_State *lua)
 	WnckScreen *screen;
 	WnckWorkspace *workspace;
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "center: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -1638,11 +1726,11 @@ int c_center(lua_State *lua)
 
 	workspace = wnck_screen_get_active_workspace(screen);
 
-	if (workspace==NULL) {
+	if (workspace == NULL) {
 		workspace = wnck_screen_get_workspace(screen,0);
 	}
 
-	if (workspace==NULL) {
+	if (workspace == NULL) {
 		g_printerr(_("Could not get workspace"));
 		lua_pushboolean(lua, FALSE);
 		return 1;
@@ -1680,13 +1768,13 @@ int c_set_opacity(lua_State *lua)
 	int top = lua_gettop(lua);
 	//WnckScreen *screen;
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua, "set_opacity: %s", one_indata_expected_error);
 		return 0;
 	}
 
-	int type=lua_type(lua, 1);
-	if (type!=LUA_TNUMBER) {
+	int type = lua_type(lua, 1);
+	if (type != LUA_TNUMBER) {
 		luaL_error(lua, "set_opacity: %s", number_expected_as_indata_error);
 		return 0;
 	}
@@ -1710,13 +1798,13 @@ int c_set_window_type(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=1) {
+	if (top != 1) {
 		luaL_error(lua, "set_window_type: %s", one_indata_expected_error);
 		return 0;
 	}
 
-	int type=lua_type(lua, 1);
-	if (type!=LUA_TSTRING) {
+	int type = lua_type(lua, 1);
+	if (type != LUA_TSTRING) {
 		luaL_error(lua, "set_window_type: %s", string_expected_as_indata_error);
 		return 0;
 	}
@@ -1735,13 +1823,13 @@ int c_set_window_type(lua_State *lua)
 
 
 /**
- * return the geometry of the screen  to the LUA script
+ * return the geometry of the screen  to the Lua script
  */
 int c_get_screen_geometry(lua_State *lua)
 {
 	int top = lua_gettop(lua);
 
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "get_screen_geometry: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -1769,7 +1857,7 @@ int c_get_screen_geometry(lua_State *lua)
 int c_focus(lua_State *lua)
 {
 	int top = lua_gettop(lua);
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "focus: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -1790,7 +1878,7 @@ int c_focus(lua_State *lua)
 int c_close_window(lua_State *lua)
 {
 	int top = lua_gettop(lua);
-	if (top!=0) {
+	if (top != 0) {
 		luaL_error(lua, "close_window: %s", no_indata_expected_error);
 		return 0;
 	}
@@ -1814,19 +1902,194 @@ int c_get_window_fullscreen(lua_State *lua)
 		luaL_error(lua, "is_fullscreen: %s", no_indata_expected_error);
 		return 0;
 	}
-	
+
 	gboolean result = FALSE;
-	
+
 	WnckWindow *window = get_current_window();
 	if (window) {
 		result = wnck_window_is_fullscreen(window);
 	}
-	
+
 	lua_pushboolean(lua, result);
-	
+
 	return 1;
 }
 
+
+/**
+ *
+ */
+int c_xy(lua_State *lua)
+{
+	int top = lua_gettop(lua);
+
+	if (top == 0) {
+		// return the xy coordinates of the window
+
+		WnckWindow *window = get_current_window();
+		if (window != NULL) {
+
+			int x, y, width, height;
+
+			wnck_window_get_geometry(window, &x, &y, &width, &height);
+
+			lua_pushnumber(lua, x);
+			lua_pushnumber(lua, y);
+
+			return 2;
+		}
+
+	} else if (top == 2) {
+		// set the coordinates of the window
+
+		int type1 = lua_type(lua, 1);
+		int type2 = lua_type(lua, 2);
+
+		if ((type1 != LUA_TNUMBER) ||
+		        (type2 != LUA_TNUMBER)) {
+			luaL_error(lua, "xy: %s", two_indata_expected_error);
+			return 0;
+		}
+
+		int x = lua_tonumber(lua, 1);
+		int y = lua_tonumber(lua, 2);
+
+		if (!devilspie2_emulate) {
+
+			WnckWindow *window = get_current_window();
+
+			if (window) {
+				wnck_window_set_geometry(window,
+				                         WNCK_WINDOW_GRAVITY_CURRENT,
+				                         WNCK_WINDOW_CHANGE_X + WNCK_WINDOW_CHANGE_Y,
+				                         x, y, -1, -1);
+			}
+		}
+
+	} else {
+		luaL_error(lua, "xy: %s", two_indata_expected_error);
+		return 0;
+	}
+	return 0;
+}
+
+
+/**
+ *
+ */
+int c_xywh(lua_State *lua)
+{
+	int top = lua_gettop(lua);
+
+	if (top == 0) {
+		// Return the xywh settings of the window
+
+		WnckWindow *window = get_current_window();
+		if (window != NULL) {
+
+			int x, y, width, height;
+
+			wnck_window_get_geometry(window, &x, &y, &width, &height);
+
+			lua_pushnumber(lua, x);
+			lua_pushnumber(lua, y);
+			lua_pushnumber(lua, width);
+			lua_pushnumber(lua, height);
+
+			return 4;
+		}
+
+	} else if (top == 4) {
+		// Set the xywh settings in the window
+
+
+		int type1 = lua_type(lua, 1);
+		int type2 = lua_type(lua, 2);
+		int type3 = lua_type(lua, 3);
+		int type4 = lua_type(lua, 4);
+
+		if ((type1 != LUA_TNUMBER) ||
+		        (type2 != LUA_TNUMBER) ||
+		        (type3 != LUA_TNUMBER) ||
+		        (type4 != LUA_TNUMBER)) {
+			luaL_error(lua, "xywh: %s", four_indata_expected_error);
+			return 0;
+		}
+
+		int x = lua_tonumber(lua, 1);
+		int y = lua_tonumber(lua, 2);
+		int xsize = lua_tonumber(lua, 3);
+		int ysize = lua_tonumber(lua, 4);
+
+		if (!devilspie2_emulate) {
+			WnckWindow *window = get_current_window();
+
+			set_window_geometry(window, x, y, xsize, ysize);
+		}
+
+		return 0;
+
+	} else {
+		luaL_error(lua, "xywh: %s", four_indata_expected_error);
+		return 0;
+	}
+
+	return 0;
+}
+
+struct lua_callback {
+	lua_State *lua;
+	int ref;
+};
+
+static void on_geometry_changed(WnckWindow *window, struct lua_callback *callback)
+{
+	if (callback == NULL)
+		return;
+
+	WnckWindow *old_window = get_current_window();
+	set_current_window(window);
+
+	lua_rawgeti(callback->lua, LUA_REGISTRYINDEX, callback->ref);
+	lua_pcall(callback->lua, 0, 0, 0);
+
+	set_current_window(old_window);
+}
+
+static void on_geometry_changed_disconnect(gpointer data, GClosure *closure)
+{
+	g_free(data);
+}
+
+/**
+ *
+ */
+int c_on_geometry_changed(lua_State *lua)
+{
+	int top = lua_gettop(lua);
+	if (top != 1) {
+		luaL_error(lua, "on_geometry_changed: %s", one_indata_expected_error);
+		return 0;
+	}
+
+	if (lua_type(lua, 1) != LUA_TFUNCTION) {
+		luaL_error(lua, "on_geometry_changed: %s", "function expected");
+		return 0;
+	}
+
+	struct lua_callback *cb = g_malloc(sizeof(struct lua_callback));
+	cb->lua = lua;
+	cb->ref = luaL_ref(lua, LUA_REGISTRYINDEX);
+
+	WnckWindow *window = get_current_window();
+
+	if (window) {
+		g_signal_connect_data(window, "geometry-changed", G_CALLBACK(on_geometry_changed), (gpointer)cb, (GClosureNotify)(on_geometry_changed_disconnect), 0);
+	}
+
+	return 0;
+}
+
 /*
  * Devilspie:
 
